@@ -178,8 +178,7 @@ class Collection(object):
         docs = doc_or_docs
         if isinstance(docs, types.DictType):
             docs = [docs]
-
-        if not isinstance(docs, types.ListType):
+        elif not isinstance(docs, types.ListType):
             raise TypeError("insert takes a document or list of documents")
 
         if manipulate:
@@ -261,7 +260,7 @@ class Collection(object):
         self._send_message(2006, _ZERO + bson.BSON.from_dict(spec))
 
     def find_one(self, spec_or_object_id=None, fields=None, slave_okay=None,
-                 _sock=None):
+                 _sock=None, _must_use_master=False):
         """Get a single object from the database.
 
         Raises TypeError if the argument is of an improper type. Returns a
@@ -286,7 +285,8 @@ class Collection(object):
             spec = SON({"_id": spec})
 
         for result in self.find(spec, limit=-1, fields=fields,
-                                slave_okay=slave_okay, _sock=_sock):
+                                slave_okay=slave_okay, _sock=_sock,
+                                _must_use_master=_must_use_master):
             return result
         return None
 
@@ -309,7 +309,7 @@ class Collection(object):
 
     def find(self, spec=None, fields=None, skip=0, limit=0,
              slave_okay=None, timeout=True, snapshot=False,
-             _sock=None):
+             _sock=None, _must_use_master=False):
         """Query the database.
 
         The `spec` argument is a prototype document that all results must
@@ -333,8 +333,7 @@ class Collection(object):
             in the result set ("_id" will always be included)
           - `skip` (optional): the number of documents to omit (from the start
             of the result set) when returning the results
-          - `limit` (optional): the maximum number of results to return in the
-            first reply message, or 0 for the default return size
+          - `limit` (optional): the maximum number of results to return
           - `slave_okay` (optional): if True, this query should be allowed to
             execute on a slave (by default, certain queries are not allowed to
             execute on mongod instances running in slave mode). If slave_okay
@@ -377,7 +376,7 @@ class Collection(object):
             fields = self._fields_list_to_dict(fields)
 
         return Cursor(self, spec, fields, skip, limit, slave_okay, timeout, snapshot,
-                      _sock=_sock)
+                      _sock=_sock, _must_use_master=_must_use_master)
 
     def count(self):
         """Get the number of documents in this collection.
@@ -554,7 +553,8 @@ class Collection(object):
     #
     # Waiting on this because group command support for CodeWScope
     # wasn't added until 1.1
-    def group(self, keys, condition, initial, reduce, command=False):
+    def group(self, keys, condition, initial, reduce, finalize=None,
+              command=False):
         """Perform a query similar to an SQL group by operation.
 
         Returns an array of grouped items.
@@ -565,19 +565,33 @@ class Collection(object):
             query specification)
           - `initial`: initial value of the aggregation counter object
           - `reduce`: aggregation function as a JavaScript string
+          - `finalize`: function to be called on each object in output list.
           - `command` (optional): if True, run the group as a command instead
             of in an eval - it is likely that this option will eventually be
-            deprecated and all groups will be run as commands
+            deprecated and all groups will be run as commands. Please only use
+            as a keyword argument, not as a positional argument.
         """
+
+        #for now support people passing command in its old position
+        if finalize in (True, False):
+            command = finalize
+            finalize = None
+            warnings.warn("Please only pass 'command' as a keyword argument."
+                         ,DeprecationWarning)
+
         if command:
             if not isinstance(reduce, Code):
                 reduce = Code(reduce)
-            return self.__database._command({"group":
-                                                 {"ns": self.__collection_name,
-                                                  "$reduce": reduce,
-                                                  "key": self._fields_list_to_dict(keys),
-                                                  "cond": condition,
-                                                  "initial": initial}})["retval"]
+            group = {"ns": self.__collection_name,
+                    "$reduce": reduce,
+                    "key": self._fields_list_to_dict(keys),
+                    "cond": condition,
+                    "initial": initial}
+            if finalize is not None:
+                if not isinstance(finalize, Code):
+                    finalize = Code(finalize)
+                group["finalize"] = finalize
+            return self.__database._command({"group":group})["retval"]
 
         scope = {}
         if isinstance(reduce, Code):
@@ -591,6 +605,7 @@ class Collection(object):
     var c = db[ns].find(condition);
     var map = new Map();
     var reduce_function = %s;
+    var finalize_function = %s; //function or null
     while (c.hasNext()) {
         var obj = c.next();
 
@@ -608,8 +623,18 @@ class Collection(object):
         }
         reduce_function(obj, aggObj);
     }
-    return {"result": map.values()};
-}""" % reduce
+
+    out = map.values();
+    if (finalize_function !== null){
+        for (var i=0; i < out.length; i++){
+            var ret = finalize_function(out[i]);
+            if (ret !== undefined)
+                out[i] = ret;
+        }
+    }
+
+    return {"result": out};
+}""" % (reduce, (finalize or 'null'));
         return self.__database.eval(Code(group_function, scope))["result"]
 
     def rename(self, new_name):
