@@ -447,7 +447,7 @@ class Connection(object): # TODO support auth for pooling
         request_id = self.__increment_id()
         to_send = struct.pack("<i", 16 + len(data))
         to_send += struct.pack("<i", request_id)
-        to_send += struct.pack("<i", 0) # responseTo
+        to_send += "\x00\x00\x00\x00" # responseTo
         to_send += struct.pack("<i", operation)
 
         to_send += data
@@ -509,7 +509,9 @@ class Connection(object): # TODO support auth for pooling
 
     __hack_socket_lock = threading.Lock()
 
-    def _receive_message(self, operation, data, _sock=None):
+    # we just ignore _must_use_master here: it's only relavant for
+    # MasterSlaveConnection instances.
+    def _receive_message(self, operation, data, _sock=None, _must_use_master=False):
         """Receive a message from Mongo.
 
         Sends the given message and returns the response.
