@@ -37,7 +37,7 @@ class Cursor(object):
     """
 
     def __init__(self, collection, spec, fields, skip, limit, slave_okay,
-                 timeout, snapshot=False, _sock=None):
+                 timeout, snapshot=False, _sock=None, _must_use_master=False):
         """Create a new cursor.
 
         Should not be called directly by application developers.
@@ -54,6 +54,7 @@ class Cursor(object):
         self.__explain = False
         self.__hint = None
         self.__socket = _sock
+        self.__must_use_master = _must_use_master
 
         self.__data = []
         self.__id = None
@@ -173,15 +174,73 @@ class Cursor(object):
         :Parameters:
           - `skip`: the number of results to skip
         """
-        if not isinstance(skip, types.IntType):
+        if not isinstance(skip, (types.IntType, types.LongType)):
             raise TypeError("skip must be an int")
         self.__check_okay_to_chain()
 
         self.__skip = skip
         return self
 
+    def __getitem__(self, index):
+        """Get a single document or a slice of documents from this cursor.
+
+        Raises InvalidOperation if this cursor has already been used.
+
+        To get a single document use an integral index, e.g.:
+
+        >>> db.test.find()[50]
+
+        An IndexError will be raised if the index is negative or greater than
+        the amount of documents in this cursor.
+
+        To get a slice of documents use a slice index, e.g.:
+
+        >>> db.test.find()[20:25]
+
+        This will return this cursor with a limit of 5 and skip of 20 applied.
+        Using a slice index will override any prior limits or skips applied to
+        this cursor (including those applied through previous calls to this
+        method). Raises IndexError when the slice has a step, a negative start
+        value, or a stop value less than or equal to the start value.
+
+        :Parameters:
+          - `index`: An integer or slice index to be applied to this cursor
+        """
+        self.__check_okay_to_chain()
+        if isinstance(index, types.SliceType):
+            if index.step is not None:
+                raise IndexError("Cursor instances do not support slice steps")
+
+            skip = 0
+            if index.start is not None:
+                if index.start < 0:
+                    raise IndexError("Cursor instances do not support negative indices")
+                skip = index.start
+
+            if index.stop is not None:
+                limit = index.stop - skip
+                if limit <= 0:
+                    raise IndexError("stop index must be greater than start index for slice %r" % index)
+            else:
+                limit = 0
+
+            self.__skip = skip
+            self.__limit = limit
+            return self
+
+        if isinstance(index, (types.IntType, types.LongType)):
+            if index < 0:
+                raise IndexError("Cursor instances do not support negative indices")
+            clone = self.clone()
+            clone.skip(index)
+            clone.limit(-1) # use a hard limit
+            for doc in clone:
+                return doc
+            raise IndexError("no such item for Cursor instance")
+        raise TypeError("index %r cannot be applied to Cursor instances" % index)
+
     def sort(self, key_or_list, direction=None):
-        """Sorts this cursors results.
+        """Sorts this cursor's results.
 
         Takes either a single key and a direction, or a list of (key,
         direction) pairs. The key(s) must be an instance of (str, unicode), and
@@ -217,6 +276,23 @@ class Cursor(object):
             return 0
         return int(response["n"])
 
+    def __len__(self):
+        """Get the number of documents in this cursor.
+
+        This method relies on count() as well as any limit or skip that has
+        been applied to this cursor to return the number of documents that will
+        actually be returned by the cursor. Changes to this cursor's limit or
+        skip values, as well as changes to the data in the database itself, can
+        cause this value to be different from the actual number of documents
+        that will be iterated by the cursor.
+        """
+        count = self.count() - self.__skip
+        if count < 0:
+            return 0
+        if self.__limit:
+            return min(count, self.__limit)
+        return count
+
     def explain(self):
         """Returns an explain plan record for this cursor.
         """
@@ -290,7 +366,8 @@ class Cursor(object):
 
         def send_message(operation, message):
             db = self.__collection.database()
-            kwargs = {"_sock": self.__socket}
+            kwargs = {"_sock": self.__socket,
+                      "_must_use_master": self.__must_use_master}
             if self.__connection_id is not None:
                 kwargs["_connection_to_use"] = self.__connection_id
 
