@@ -30,6 +30,19 @@ except: # for Python < 2.5
 from errors import InvalidId
 
 
+def _machine_and_pid_bytes():
+    """Get the machine and pid portion of an ObjectId.
+    """
+    # 3 bytes machine
+    machine_hash = _md5func()
+    machine_hash.update(socket.gethostname())
+    bytes = machine_hash.digest()[0:3]
+
+    # 2 bytes pid
+    bytes += struct.pack(">H", os.getpid() % 0xFFFF)
+    return bytes
+
+
 class ObjectId(object):
     """A Mongo ObjectId.
     """
@@ -37,6 +50,8 @@ class ObjectId(object):
     _inc = 0
     _inc_lock = threading.Lock()
 
+    _machine_and_pid_bytes = _machine_and_pid_bytes()
+
     def __init__(self, id=None):
         """Initialize a new ObjectId.
 
@@ -62,13 +77,8 @@ class ObjectId(object):
         # 4 bytes current time
         oid += struct.pack(">i", int(time.time()))
 
-        # 3 bytes machine
-        machine_hash = _md5func()
-        machine_hash.update(socket.gethostname())
-        oid += machine_hash.digest()[0:3]
-
-        # 2 bytes pid
-        oid += struct.pack(">H", os.getpid() % 0xFFFF)
+        # 3 bytes machine + 2 bytes pid
+        oid += ObjectId._machine_and_pid_bytes
 
         # 3 bytes inc
         ObjectId._inc_lock.acquire()
@@ -147,11 +157,12 @@ class ObjectId(object):
         return cls(legacy_str[7::-1] + legacy_str[:7:-1])
     from_legacy_str = classmethod(from_legacy_str)
 
-    @property
-    def binary(self):
+    def get_binary(self):
         """Get the binary representation of this ObjectId.
         """
         return self.__id
+    
+    binary = property(get_binary)
 
     def __str__(self):
         return self.__id.encode("hex")
