@@ -519,55 +519,68 @@ class TestCollection(unittest.TestCase):
         db = self.db
         db.drop_collection("test")
 
-        self.assertEqual([], db.test.group([], {},
-                                           {"count": 0},
-                                           "function (obj, prev) { "
-                                           "prev.count++; }"))
-        self.assertEqual([], db.test.group([], {},
-                                           {"count": 0},
-                                           "function (obj, prev) { "
-                                           "prev.count++; }", command=True))
+        def group_checker(args, expected, with_command=True):
+            eval = db.test.group(*args)
+            self.assertEqual(eval, expected)
+
+            if with_command:
+                cmd = db.test.group(*args, **{"command": True})
+                self.assertEqual(cmd, expected)
+
+
+        args = [[], {},
+                {"count": 0},
+                "function (obj, prev) { prev.count++; }"]
+        expected = []
+        group_checker(args, expected)
 
         db.test.save({"a": 2})
         db.test.save({"b": 5})
         db.test.save({"a": 1})
 
-        self.assertEqual(3, db.test.group([], {},
-                                          {"count": 0},
-                                          "function (obj, prev) { "
-                                          "prev.count++; }")[0]["count"])
-        self.assertEqual(3, db.test.group([], {},
-                                          {"count": 0},
-                                          "function (obj, prev) { "
-                                          "prev.count++; }",
-                                          command=True)[0]["count"])
-        self.assertEqual(1, db.test.group([],
-                                          {"a": {"$gt": 1}},
-                                          {"count": 0},
-                                          "function (obj, prev) { "
-                                          "prev.count++; }")[0]["count"])
-        self.assertEqual(1, db.test.group([],
-                                          {"a": {"$gt": 1}},
-                                          {"count": 0},
-                                          "function (obj, prev) { "
-                                          "prev.count++; }",
-                                          command=True)[0]["count"])
+        args = [[], {},
+                {"count": 0},
+                "function (obj, prev) { prev.count++; }"]
+        expected = [{'count': 3}]
+        group_checker(args, expected)
+
+        args = [[],
+                {"a": {"$gt": 1}},
+                {"count": 0},
+                "function (obj, prev) { prev.count++; }"]
+        expected = [{'count': 1}]
+        group_checker(args, expected)
 
         db.test.save({"a": 2, "b": 3})
 
+        args = [["a"], {},
+                {"count": 0},
+                "function (obj, prev) { prev.count++; }"]
         # NOTE maybe we can't count on this ordering being right
         expected = [{"a": 2, "count": 2},
                     {"a": None, "count": 1},
                     {"a": 1, "count": 1}]
-        self.assertEqual(expected, db.test.group(["a"], {},
-                                                 {"count": 0},
-                                                 "function (obj, prev) { "
-                                                 "prev.count++; }",
-                                                 command=True))
-        self.assertEqual(expected, db.test.group(["a"], {},
-                                                 {"count": 0},
-                                                 "function (obj, prev) { "
-                                                 "prev.count++; }"))
+        group_checker(args, expected)
+
+        # modifying finalize
+        args = [["a"], {},
+                {"count": 0},
+                "function (obj, prev) { prev.count++; }",
+                "function(obj){obj.count++;}"]
+        expected = [{"a": 2, "count": 3},
+                    {"a": None, "count": 2},
+                    {"a": 1, "count": 2}]
+        group_checker(args, expected, with_command=version.at_least(db.connection(), (1, 1)))
+
+        # returning finalize
+        args = [["a"], {},
+                {"count": 0},
+                "function (obj, prev) { prev.count++; }",
+                "function(obj){ return obj.count;}"]
+        expected = [2, # a:2
+                    1, # a:None
+                    1] # a:1
+        group_checker(args, expected, with_command=version.at_least(db.connection(), (1, 1)))
 
         self.assertRaises(OperationFailure, db.test.group, [], {}, {}, "5 ++ 5")
         self.assertRaises(OperationFailure, db.test.group, [], {}, {}, "5 ++ 5", command=True)
@@ -617,10 +630,13 @@ class TestCollection(unittest.TestCase):
         self.assertEqual(2000, db.test.count())
 
         i = 0
-        for _ in db.test.find(limit=1900):
+        y = 0
+        for doc in db.test.find(limit=1900):
             i += 1
+            y += doc["x"]
 
         self.assertEqual(1900, i)
+        self.assertEqual(1804050, y)
 
     def test_find_kwargs(self):
         db = self.db
